/************************************************************************
 *   IRC - Internet Relay Chat, ircd/s_serv.c (formerly ircd/s_msg.c)
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <utmp.h>
#include <string.h>
#include "h.h"

static char buf[BUFSIZE];


/*
** m_functions execute protocol messages on this server:
**
**	cptr	is always NON-NULL, pointing to a *LOCAL* client
**		structure (with an open socket connected!). This
**		identifies the physical socket where the message
**		originated (or which caused the m_function to be
**		executed--some m_functions may call others...).
**
**	sptr	is the source of the message, defined by the
**		prefix part of the message if present. If not
**		or prefix not found, then sptr==cptr.
**
**		(!IsServer(cptr)) => (cptr == sptr), because
**		prefixes are taken *only* from servers...
**
**		(IsServer(cptr))
**			(sptr == cptr) => the message didn't
**			have the prefix.
**
**			(sptr != cptr && IsServer(sptr) means
**			the prefix specified servername. (?)
**
**			(sptr != cptr && !IsServer(sptr) means
**			that message originated from a remote
**			user (not local).
**
**		combining
**
**		(!IsServer(sptr)) means that, sptr can safely
**		taken as defining the target structure of the
**		message in this server.
**	*Always* true (if 'parse' and others are working correct):
**
**	1)	sptr->from == cptr  (note: cptr->from == cptr)
**
**	2)	MyConnect(sptr) <=> sptr == cptr (e.g. sptr
**		*cannot* be a local connection, unless it's
**		actually cptr!). [MyConnect(x) should probably
**		be defined as (x == x->from) --msa ]
**
**	parc	number of variable parameter strings (if zero,
**		parv is allowed to be NULL)
**
**	parv	a NULL terminated list of parameter pointers,
**
**			parv[0], sender (prefix string), if not present
**				this points to an empty string.
**			parv[1]...parv[parc-1]
**				pointers to additional parameters
**			parv[parc] == NULL, *always*
**
**		note:	it is guaranteed that parv[0]..parv[parc-1] are all
**			non-NULL pointers.
*/
extern aConfItem *find_tline PROTO ((aClient *));

/*
 * m_motd
 * parv[0] - sender prefix
 * parv[1] - servername
 *
 * modified 30 mar 1995 by flux (cmlambertus@ucdavis.edu)
 * T line patch - display motd based on hostmask
 */
int
m_motd (cptr, sptr, parc, parv)
     aClient *cptr, *sptr;
     int parc;
     char *parv[];
{
  int fd, nr, tflag;
  char line[80];
  Reg1 char *tmp;
  struct stat sb;
  struct tm *tm;
  aConfItem *ftmp;

  if (check_registered (sptr))
    return 0;

  if (hunt_server (cptr, sptr, ":%s MOTD :%s", 1, parc, parv) != HUNTED_ISME)
    return 0;

  /*
   * find out if we have a T line for our hostname
   */
  ftmp = find_tline (cptr);

  /*
   * stop NFS hangs...most systems should be able to open a file in
   * 3 seconds. -avalon (curtesy of wumpus)
   */
  (void) alarm (3);
  /* if we returned 0 in find_tline, there is no T line for this host
   * so display the stock ircd.motd
   */
  if (!ftmp || (fd = open (ftmp->passwd, O_RDONLY)) == -1)
    fd = open (MOTD, O_RDONLY);
  (void) alarm (0);
  if (fd == -1)
    {
      sendto_one (sptr, err_str (ERR_NOMOTD), me.name, parv[0]);
      return 0;
    }
  (void) fstat (fd, &sb);
  sendto_one (sptr, rpl_str (RPL_MOTDSTART), me.name, parv[0], me.name);
  tm = localtime ((time_t *) & sb.st_mtime);	/* NetBSD needs cast */
  sendto_one (sptr, ":%s %d %s :- %d/%d/%d %d:%02d", me.name, RPL_MOTD,
	      parv[0], tm->tm_mday, tm->tm_mon + 1, 1900 + tm->tm_year,
	      tm->tm_hour, tm->tm_min);
  (void) dgets (-1, NULL, 0);	/* make sure buffer is at empty pos */
  while ((nr = dgets (fd, line, sizeof (line) - 1)) > 0)
    {
      line[nr] = '\0';
      if ((tmp = (char *) index (line, '\n')))
	*tmp = '\0';
      if ((tmp = (char *) index (line, '\r')))
	*tmp = '\0';
      sendto_one (sptr, rpl_str (RPL_MOTD), me.name, parv[0], line);
    }
  (void) dgets (-1, NULL, 0);	/* make sure buffer is at empty pos */
  sendto_one (sptr, rpl_str (RPL_ENDOFMOTD), me.name, parv[0]);
  (void) close (fd);
  return 0;
}
